﻿qx.Mixin.define("wisej.web.MRollPatch", {

	members: {

        /**
         * Roll event handler
         *
         * @param e {qx.event.type.Roll} Roll event
         */
        _onRoll: function (e) {
            // only wheel and touch
            if (e.getPointerType() == "mouse") {
                return;
            }

            if (this._cancelRoll && e.getMomentum()) {
                e.stopMomentum();
                this._cancelRoll = null;
                return;
            }
            this._cancelRoll = null;

            var showX = this._isChildControlVisible("scrollbar-x");
            var showY = this._isChildControlVisible("scrollbar-y");

            var scrollbarY = showY ? this.getChildControl("scrollbar-y", true) : null;
            var scrollbarX = showX ? this.getChildControl("scrollbar-x", true) : null;

            var deltaY = e.getDelta().y;
            var deltaX = e.getDelta().x;

            var endY = !showY;
            var endX = !showX;

            // y case
            if (scrollbarY) {
                if (deltaY !== 0) {
                    scrollbarY.scrollBy(parseInt(deltaY, 10));
                }

                var position = scrollbarY.getPosition();
                var max = scrollbarY.getMaximum();

                // pass the event to the parent if the scrollbar is at an edge
                if (deltaY < 0 && position <= 0 || deltaY > 0 && position >= max) {
                    endY = true;
                }
            }

            // x case
            if (scrollbarX) {
                if (deltaX !== 0) {
                    scrollbarX.scrollBy(parseInt(deltaX, 10));
                }

                var position = scrollbarX.getPosition();
                var max = scrollbarX.getMaximum();
                // pass the event to the parent if the scrollbar is at an edge
                if (deltaX < 0 && position <= 0 || deltaX > 0 && position >= max) {
                    endX = true;
                }
            }

            // @ITG:Wisej: Stop the momentum only if we have the scrollbars, or it may stop an ongoing roll on the parent.
            if ((endX && scrollbarX) || (endY && scrollbarY)) {
                e.stopMomentum();
            }

            // @ITG:Wisej: Don't pass the event up when the widget can scroll, or we get a weird double scroll.
            //// pass the event to the parent if both scrollbars are at the end
            if (scrollbarX && deltaX !== 0 && !endX) {
                e.stop();
            }
            else if (scrollbarY && deltaY !== 0 && !endY) {
                e.stop();
            }
            //    (!endX && deltaY === 0) ||
            //    ((!endX || !endY ) && deltaX !== 0 && deltaY !== 0)) {
            //  // Stop bubbling and native event only if a scrollbar is visible
            //  e.stop();
            //}
        }
	}
});

qx.Class.patch(qx.ui.core.scroll.AbstractScrollArea, wisej.web.MRollPatch);